function [X,Y]=Gnurbs1(p,P,W)
xp=P(1,:);
yp=P(2,:);
wx=W(1,:);
wy=W(2,:);
Xp=xp.*wx;
Yp=yp.*wy;

[Wxy,~,KV1]=bspline_product(p,p,wx,wx,wy,wy);
[XW,YW,KV2]=bspline_product(p,p,Xp,Yp,wy,wx);
X=XW./Wxy;
Y=YW./Wxy;
Gnurbs(p,xp,yp,wx,wy)
plot(xp,yp,'o')
plot(X,Y,'Color',[0.5 0.5 1],'marker','o','MarkerSize',8,'markerfacecolor',[0.5 0.3 1],'MarkerEdgeColor','k','LineWidth',1.5);
% figure
[Cx,Cy,~]=bezierplot(2*p,X,Y,KV1);
% plot(Cx,Cy);
hold on
% plot(X,Y,'r','marker','o');

function [FX,FY,KV]=bspline_product(p,q,fx,fy,gx,gy)
[Fx,Fy,KV1,bsf]=bezier_extract(p,fx,fy);
[Gx,Gy,KV2,bsg]=bezier_extract(q,gx,gy);

for i=1:min(bsf,bsg)
    Fxx=Fx(p*(i-1)+1:p*i+1);
    Fyy=Fy(p*(i-1)+1:p*i+1);
    Gxx=Gx(q*(i-1)+1:q*i+1);
    Gyy=Gy(q*(i-1)+1:q*i+1);
    hx=bezierproduct(p,q,Fxx,Gxx);
    hy=bezierproduct(p,q,Fyy,Gyy);
    Hx((p+q)*(i-1)+1:(p+q)*i+1)=hx;
    Hy((p+q)*(i-1)+1:(p+q)*i+1)=hy;
end



K=sort([KV1,KV2]);
K=K(2:size(K,2)-1);
mult=max(p+1,q+1);
[FX,FY,KV]=knot_removal(p+q,Hx,Hy,K,mult);

function Gnurbs(p,xN,yN,Wx,Wy)
%this program produces Nurbs curves with different degree and control
%points

N=size(xN,2);

M=N+p+1;   %(number of knots)(m+1)
mp=(M)-2*(p+1); %number of middle points in knot vector
%defining knot vector:in this vector using clamped condition first (p+1)
%knots are zero and last(p+1) knots are one, and middle knots are
%distributed equally //////////
KV=zeros(1,M); %knot vector
for i=1:p+1
    KV(1,i)=0;
    KV(1,M-(i-1))=1;
end
for i=1:mp
    KV(1,p+1+i)=i*(1-0)/(mp+1);
end
%/////////////////////////////
U=KV;
count=0;
%evaluating basis functions for different values of parameter u
for u=0:.01:1
    count=count+1;
 NN=zeros(M-1,p+1);  %matrix of bezier functions
%defining zero order basis functions(N0,0 , N1,0 , N2,0 , ...) which is the
%first column of NN matrix ////
for i=1:M-1;
    NN(i,1)=1*(stepp(u,U(i))-stepp(u,U(i+1)));
end
%/////////////////
   
    for j=1:p
        for i=0:(M-2)-j
        A=(u-U(i+1))/(U(i+j+1)-U(i+1));
        if isnan(A) || isinf(A)
            A=0;
        end
        B=(U(i+j+1+1)-u)/(U(i+j+1+1)-U(i+1+1));
        if isnan(B)|| isinf(B)
            B=0;
        end
        NN(i+1,j+1)=A*NN(i+1,j-1+1)+B*NN(i+1+1,j-1+1);

        end
    end
    
    %calculating rational basis functions(Ri,p)
    demx=0; %the deminator of the rational basis functions
    demy=0;
%     demz=0;
    for i=1:N
        demx=demx+NN(i,p+1)*Wx(i);
        demy=demy+NN(i,p+1)*Wy(i);
%         demz=demz+NN(i,p+1)*Wz(i);
    end
        
%calculating x and y coordinate of points on curve by multiplying basis 
%function in last column of NN matrix by x and y coordinate of control
%poins:
    cx=0;
    cy=0;
%     cz=0;
    for i=1:N
        cx=cx+NN(i,p+1)*Wx(i)/demx*xN(i);  %temp variable Ri,p=NN(i,p+1)*W(i)/dem
        cy=cy+NN(i,p+1)*Wy(i)/demy*yN(i);  %temp variable
%         cz=cz+NN(i,p+1)*Wz(i)/demz*zN(i);
    end
    Cx(count)=cx;     %x coordinate of points on curve for
                      %specified parameter u 
    Cy(count)=cy;     %y coordinate of points on curve for specified parameter u 
%     Cz(count)=cz;     %z coordinate of points on curve for specified parameter u 
    
end
% figure
% plot(Cx,Cy)       
% hold on
% plot(xN,yN,'r')

function [h] = bezierproduct(p,q,f,g)
%this function takse the control points coordinates and degree of two
%bezier curves and calculates the product of these two curves
%p,q: degree of the two curves
%f,g: control points of the two curves
%h: control points of the result curve
nf=size(f,2);
ng=size(g,2);
h = zeros(1,nf+ng-1);
first=0;
% last=nf+ng-2;
last=p+q;
for i = first:last
    h(i+1) = 0;
    for j = max(0,i-q):min(p,i)
        h(i+1) = h(i+1) + nchoosek(p,j)*nchoosek(q,i-j)/nchoosek(p+q,i)*f(j+1)*g(i-j+1);
    end
end


function [xp,yp,KV,bs]=bezier_extract(p,xp,yp)
%this function extracts the bezier sections of a spline curve by adding appropriate number of knots
%if the multiplicity of a knot is equal to curve degree, the control point
%is located on the curve (in a bezire curve, the multiplicity of first and
%last knot is equal to p+1 and the multiplicity of internal knots is equal
%to p)
% clc
% clear all
% close all
% p=3;
% xp=[1,-1,3,5,6,8,10,12,14,12];
% yp=[2,3,4,4.2,3.5,3,3,2,2.5,3];
% xp=10*sort(randn(1,8));
% yp=10*randn(1,8);
M=size(xp,2);
[Cx,Cy,KV]=bezierplot(p,xp,yp,0);
N=size(KV,2);
% plot(xp,yp,'o')
RP=rpt(KV);
m=size(RP,2);
i=1;
bs=1; %number of bezier sections
while min(RP)<p
    
    if RP(i)<p
        mult=p-RP(i); %number of knots to be added
        count2=1;
      bs=bs+1;
        for j=1:mult
            s=RP(i);    %multiplicity of the left knot 
            k=sum(RP(1:i));  % the index of the knot value, after which the new knots to be added
            Min=k-p+j;        %the initial index of the knot to be added
            Max=k-s ;         % the final index of the knot to be added  
            count=1;
            if j==1
                t1=Min;
                t2=Max;
                %defining matrix of control points to be added, the first
                %column is the initial control points
            Xp=zeros(mult+1);  
            Yp=zeros(mult+1);
            alpha=zeros(mult);
            %initializing the first row of Xp and Yp matrices
            Xp(1,:)=xp(Min-1:Max);
            Yp(1,:)=yp(Min-1:Max);
            
            end
            
            for ii=Min:Max                
                alpha(count,j)=(KV(k)-KV(ii))/(KV(ii+p-j+1)-KV(ii));
                Xp(count2+1,ii-Min+count2+1)=alpha(count,j)*Xp(count2,ii-Min+count2+1)+(1-alpha(count,j))*Xp(count2,ii-Min+count2);
               Yp(count2+1,ii-Min+count2+1)=alpha(count,j)*Yp(count2,ii-Min+count2+1)+(1-alpha(count,j))*Yp(count2,ii-Min+count2);
               count=count+1;               
            end
            count2=count2+1;
                       
        end
        
%choosing the appropriate control points from Xp and Yp matrices to be added to available control points        
x1=diag(Xp);
x2=Xp(mult:-1:2,mult+1);
Xadd=[x1(2:mult+1)',x2'];

y1=diag(Yp);
y2=Yp(mult:-1:2,mult+1);
Yadd=[y1(2:mult+1)',y2'];
            
xp=[xp(1:t1-1),Xadd,xp(t2:M)];
yp=[yp(1:t1-1),Yadd,yp(t2:M)];    
KV=[KV(1:k),KV(k)*ones(1,mult),KV(k+1:N)];
    else
        i=i+1;
    end
    M=size(xp,2);
    N=size(KV,2);
    RP=rpt(KV);
    m=size(RP,2);
end

function [Fx,Fy,KV1]=knot_removal(p,Fx,Fy,KV1,km)
N=size(Fx,2);
M=size(KV1,2);
RP=rpt(KV1);
m=size(RP,2);
i=1;
while max(RP(2:m-1))>km
    if RP(i)>km && RP(i)<p+1
        mult=RP(i);                
        s=RP(i);
        k=sum(RP(1:i));
        Min=k-p;
        Max=(2*k-p-s-1)/2;
        count=1;
        count2=1;
        if mult==p
            Px0=Fx;
            Py0=Fy;
            Px1=[Px0(1:Min-1),Px0(Min+1:N)];
            Py1=[Py0(1:Min-1),Py0(Min+1:N)];
        else
Px=[];
Py=[];
Pxj=[];
Pyj=[];
% Px2=Px1;
% Py2=Py1;
        for ii=Min:Max
%             alpha=(KV1(k)-KV1(ii))/(KV1(ii+p+RP0(i)-mult)-KV1(ii));
            alpha=(KV1(k)-KV1(ii))/(KV1(ii+p+1)-KV1(ii));
            Px1(ii)=(Px0(ii)-(1-alpha)*Px1(ii-1))/alpha;
            Py1(ii)=(Py0(ii)-(1-alpha)*Py1(ii-1))/alpha;
            Px(count)=Px1(ii);
            Py(count)=Py1(ii);
            count=count+1;
        end
        min2=(2*k-p-s+2)/2;
        max2=k-s;
        if floor(min2)~=min2
            min2=min2+0.5;
            
        end

        count=1; 
        for jj=max2:-1:min2
            alpha=(KV1(k)-KV1(jj))/(KV1(jj+p+1)-KV1(jj));
            Px1(jj)=(Px0(jj+1)-alpha*Px1(jj+1))/(1-alpha);
            Py1(jj)=(Py0(jj+1)-alpha*Py1(jj+1))/(1-alpha);
            Pxj(count)=Px1(jj);
            Pyj(count)=Py1(jj);
            count=count+1;
        end

         Px0=Px1;
         Py0=Py1;
         Px1=[Px0(1:Min-1),Px,Pxj(size(Pxj,2):-1:1),Px0(Min+p-s+1:N)];
         Py1=[Py0(1:Min-1),Py,Pyj(size(Pyj,2):-1:1),Py0(Min+p-s+1:N)];
        
        
        
        end
        KV1=[KV1(1:k-1),KV1(k+1:M)];
        
        Fx=Px1;
        Fy=Py1;
    else
    i=i+1;
    end

N=size(Fx,2);
M=size(KV1,2);
RP=rpt(KV1);
end

function [Cx,Cy,U]=bezierplot(p,xN,yN,knotv)
%this function takes x and y coordinate of control points and degree of
%B-spline polynomials and produces x and y coordinate of points on curve
%inputs: p:degree, 
%xN: x coordinate of control points
%yN: y coordinate of control points
%knotv: knot vector, if knotv=0, the knot vector is calculated in this
%program

%outputs: Cx,Cy: x and y value of points on the curve, U:knotvector



N=length(xN);   %(number of control points)(n+1)
M=N+p+1;   %(number of knots)(m+1)
mp=(M)-2*(p+1); %number of middle points in knot vector
if knotv==0
KV=zeros(1,M); %knot vector
for i=1:p+1
    KV(1,i)=0;
    KV(1,M-(i-1))=1;
end
for i=1:mp
    KV(1,p+1+i)=i*(1-0)/(mp+1);
end
else
    KV=knotv;
end

U=KV;
count=0;
for u=0:.01:1
% u=.9;
    count=count+1;
 NN=zeros(M-1,p+1);  %matrix of bezier functions
for i=1:M-1;
    NN(i,1)=1*(stepp(u,U(i))-stepp(u,U(i+1)));
end

 
    for j=1:p
        for i=0:(M-2)-j
        A=(u-U(i+1))/(U(i+j+1)-U(i+1));
        if isnan(A) || isinf(A)
            A=0;
        end
        B=(U(i+j+1+1)-u)/(U(i+j+1+1)-U(i+1+1));
        if isnan(B)|| isinf(B)
            B=0;
        end
        NN(i+1,j+1)=A*NN(i+1,j-1+1)+B*NN(i+1+1,j-1+1);
%         if isnan(NN(i+1,j+1))
%             NN(i+1,j+1)=0;
%         end
    end
    end

    
    cx=0;
    cy=0;
    for i=1:N
        cx=cx+NN(i,p+1)*xN(i);
        cy=cy+NN(i,p+1)*yN(i);
    end
    Cx(count)=cx;
    Cy(count)=cy;
    
end
%    
%         plot(Cx,Cy)
%         hold on
%         plot(xN,yN,'r')
    

function r=stepp(u,U)
if u>=U
    r=1;
else 
    r=0;
end
if u==1
  if u>U
    r=1;
else 
    r=0;
  end  
end

function repeat=rpt(a)
% this function takes a vector "a" as input and calculates the multiplicity
% of each array of this vector and returns the result in the "repeat"
% variable
m=size(a,2);
count=1;
i=1;

while i<m
   RP=1;
    temp=a(i);
    
    while a(i+1)==temp 
        RP=RP+1;
        i=i+1;
        if i==m 
        break
        end
    end;
    i=i+1;
    repeat(count)=RP;
    count=count+1;      
end
if a(m)~=a(m-1)
    repeat(count)=1;
end
        







    









